import logging

from spaceone.core.manager import BaseManager
from spaceone.file_manager.model.file_model import File

_LOGGER = logging.getLogger(__name__)


class FileManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.file_model: File = self.locator.get_model('File')

    def create_file(self, params):
        def _rollback(file_vo):
            _LOGGER.info(f'[ROLLBACK] Delete file : {file_vo.name} ({file_vo.file_id})')
            file_vo.delete()

        file_vo: File = self.file_model.create(params)

        self.transaction.add_rollback(_rollback, file_vo)

        return file_vo

    def update_file(self, params):
        self.update_file_by_vo(params, self.get_file(params['file_id']))

    def update_file_by_vo(self, params, file_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[ROLLBACK] Revert Data : {old_data["name"]} ({old_data["file_id"]})')
            file_vo.update(old_data)

        self.transaction.add_rollback(_rollback, file_vo.to_dict())

        return file_vo.update(params)

    def delete_file(self, file_id, domain_id):
        self.delete_file_by_vo(self.get_file(file_id, domain_id))

    @staticmethod
    def delete_file_by_vo(file_vo):
        file_vo.delete()

    def get_file(self, file_id, user_domains, only=None):
        if user_domains:
            return self.file_model.get(file_id=file_id, domain_id=user_domains, only=only)
        else:
            return self.file_model.get(file_id=file_id, only=only)

    def filter_files(self, **conditions):
        return self.file_model.filter(**conditions)

    def list_files(self, query):
        return self.file_model.query(**query)

    def stat_files(self, query):
        return self.file_model.stat(**query)



import base64
import os
import json
import collections

from cryptography.hazmat.primitives.ciphers.aead import AESGCM
from spaceone.core import config
from spaceone.core.handler import BaseMutationHandler
from spaceone.secret.error.custom import ERROR_WRONG_ENCRYPT_ALGORITHM

__all__ = ['EncryptMutationHandler']

_encrypt_verb = ['create', 'update_data']
_decrypt_verb = ['get_data']

_SUPPORTED_ENCRYPT_ALGORITHM = [
    'AES'
]

class EncryptMutationHandler(BaseMutationHandler):

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.encrypt_algorithm = self.config.get('ENCRYPT_ALGORITHM')
        self._check_config(encrypt_algorithm)

    def request(self, params:dict) -> dict:
        if not (self.transaction.resource == 'Secret' and self.transaction.verb in _encrypt_verb):
            return params
        
        encrypted_data, nonce = self._encrypt(params['data'])
        params['data'] = {
                'encrypted_data': encrypted_data
        }
        params['encrypted'] = True,
        params['encrypt_options'] = {
                'encrypt_algorithm': self.encrypt_algorithm,
                'nonce': nonce,
        }
        return params

    def response(self, result:dict) -> dict:
        if not (self.transaction.resource == 'Secret' and self.transaction.verb in _decrypt_verb):
            return result
        
        if result['encrypted'] is not True:
            return result

        encrypt_options = result.get('encrypt_options', {})
        encrypt_algorithm = encrypt_options.get('encrypt_algorithm')
        if encrypt_algorithm not in _SUPPORTED_ENCRYPT_ALGORITHM:
            raise ERROR_WRONG_ENCRYPT_ALGORITHM(encrypt_algorithm=str(encrypt_algorithm))

        plain_data = self._decrypt(result['data']['encrypted_data'], encrypt_options)

        result = {
            'encrypted': False,
            'encrypt_options': {},
            'data': plain_data
        }

        return result

    def _check_config(self, encrypt_algorithm:str) -> None:
        if encrypt_algorithm not in _SUPPORTED_ENCRYPT_ALGORITHM:
            raise ERROR_WRONG_ENCRYPT_ALGORITHM(encrypt_algorithm)
        
    def _dict_to_b64(self, data: dict) -> bytes:
        return base64.b64encode(json.dumps(data).encode())

    def _b64_to_dict(self, data):
        _data = data if isinstance(data, bytes) else data.encode()
        return json.loads(base64.b64decode(_data).decode())
    
    def _encrypt(self, plain_data:dict) -> (str, str):
        nonce = os.urandom(12)
        data_key = base64.b64decode(self.config.get('encrypt_key'))
        
        aesgcm = AESGCM(data_key)
        del data_key
        encrypted_data = aesgcm.encrypt(nonce, self._dict_to_b64(plain_data), None)
        
        encrypted_data_b64 = base64.b64encode(encrypted_data).decode()
        nonce_b64 = base64.b64encode(nonce).decode()
        return encrypted_data_b64, nonce_b64

    def _decrypt(self, encrypted_data:str, encrypt_options:dict) -> dict:
        nonce = base64.b64decode(encrypt_options['nonce'])
        data_key = base64.b64decode(self.config.get('encrypt_key'))
        
        aesgcm = AESGCM(data_key)
        del data_key

        plain_data = self._b64_to_dict(aesgcm.decrypt(nonce, base64.b64decode(encrypted_data), None))
        
        return plain_data



import logging

from spaceone.core import config, cache
from spaceone.core.manager import BaseManager
from spaceone.file_manager.error import *
from spaceone.file_manager.connector.file_base_connector import FileBaseConnector

_LOGGER = logging.getLogger(__name__)


class FileConnectorManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        backend = config.get_global('BACKEND', 'FileConnectorManager')
        try:
            _LOGGER.debug(f'[FileConnectorManager] Create {backend}')
            self.file_conn: FileBaseConnector = self.locator.get_connector(backend)
        except Exception as e:
            _LOGGER.error(f'[FileConnectorManager] not defined backend {backend}')
            raise ERROR_NOT_DEFINED_FILE_BACKEND(backend=backend)

    def get_upload_url(self, file_id, file_name):
        upload_url, upload_options = self.file_conn.get_upload_url(file_id, file_name)
        return upload_url, upload_options

    @cache.cacheable(key='file-manager:download-url:{domain_id}:{file_id}', expire=1800)
    def get_download_url(self, file_id, file_name, domain_id):
        download_url = self.file_conn.get_download_url(file_id, file_name)
        return download_url

    def check_file(self, file_id, file_name):
        return self.file_conn.check_file(file_id, file_name)

    def delete_file(self, file_id, file_name):
        self.file_conn.delete_file(file_id, file_name)
